# -*- coding: utf-8 -*-
""".. _admin_tabs:

Admin Tabs in change form
=========================

A templatetag to create a tabbed layout in a change_form.
It looks in ModelAdmin for method 'get_tabs'. The default 
``get_tabs`` is as follows::

  def get_tabs(self, request, obj):
     return self.tabs

that means you can just set :attr:`tabs` in you ModelAdmin.
Tabs is a tuple of tuples. An example of a hypothetical 
Author model whose Inlines are ``BookInline`` and ``ArticleInline``::

  tabs = (
    (_('Author'), {'items': [_('general_info')]}),
    (_('Books/Articles'), {'items': [BookModel, ArticleModel], 'active': True),
  )

Resulting in 2 tabs with label 'Author' and 'Book/Articles'
Each element of tabs is a tuple composed of:

* a label

* a list of elements (possibly empty). Each element can be eather a
  ``string`` that will be interpreted as the name of a fieldset declared in
  fieldsets or a ``model``, whose inline must be declared in the standard
  way. You can also leave an empty list: the first one that is left as an
  empty **list** will be filled with all fieldsets as in::

        (_('Author'), {}),

This grants the ability to place all peaces in any fancy way

"""

from django.template import Library

register = Library()

@register.inclusion_tag("jmb/admin_tabs.html", takes_context = True)
def create_tabs(context, adminform):

    request = context['request']    
    obj = context.get('original')
    
    if hasattr(adminform.model_admin, 'get_tabs'):
        tabs = adminform.model_admin.get_tabs(obj)
    else:
        tabs = getattr(adminform.model_admin, 'tabs', {})

    adminform.has_tabs = bool(tabs)
    if not tabs:
        return ''

    ## let's fill a tab if that's empty so that you can set
    for tab_name, item_dict in tabs:
        if not item_dict:
            item_dict['items'] = []
        item_list = item_dict.get('items')
        if isinstance(item_list, list):
            for formset in adminform:
                item_list += [formset.name]
            break

    ## let's set active, if no tab is already active
    active_tab_exists = False
    for tab_name, item_dict in tabs:
        if item_dict.get('active'):
            active_tab_exists = True
    if not active_tab_exists:
        tabs[0][1]['active'] = True

    return {
        'tabs' : tabs,
        'adminform': adminform,
        'inline_admin_formsets': context.get('inline_admin_formsets')
    }

# These filters are just utils to simplify the admin_tabs.html
# they are probably not af any broader use

@register.filter
def is_fieldset(item):

    return isinstance(item, basestring)

@register.filter
def get_fieldset_by_name(adminform, name):
    
    for fieldset in adminform:
        if fieldset.name == name:
            return fieldset

@register.filter
def get_inline_formset_by_model(inline_admin_formsets, model):

    for inline_admin_formset in inline_admin_formsets:
        # in tabs we define a list of models that are callable
        # django template system will call them so we need to
        # use it's __class__ to match it
        if inline_admin_formset.opts.model == model.__class__:
            return inline_admin_formset

@register.filter
def to_class(value):
    if value:
        return 'active'
    return ''
